import logging
from abc import ABC, abstractmethod
import subprocess
from multiprocessing import Process
from threading import Thread
import psutil
from llanta_manager.mocks.llanta_mock import LlantaMock

from dataclasses import dataclass, field
import time

_logger = logging.getLogger(__name__)


@dataclass
class Llanta:
    llanta_id: bytes = None
    nta_id: bytes = None
    process: psutil.Process = None
    ready: bool = False
    shutdown_requested: bool = False
    last_message: bytes = None
    last_contacted: int = field(default_factory=time.time)
    subnets_requested: bool = False
    last_subnets_request: int = field(default_factory=time.time)
    active_subnets: list = field(default_factory=list)

    def set_ready(self, ready: bool):
        """Setter for the "ready" field, so we can also touch last_contacted"""
        self.ready = ready
        self.last_contacted = time.time()

    def set_subnets_requested(self, subnets_requested: bool):
        """Setter for the "subnets_requested" field, so we can also touch last_subnets_request"""
        self.subnets_requested = subnets_requested
        self.last_subnets_request = time.time()


class BaseSpawner(ABC):
    """Llanta spawner interface"""

    @abstractmethod
    def spawn_llanta(
        self,
        llanta_id: bytes,
        llanta_manager_endpoint_host: str,
        llanta_manager_endpoint_port: int,
        nta_alerts_endpoint_host: str,
        nta_alerts_endpoint_port: int,
        last_message: bytes,
    ) -> Llanta:
        pass


class LlantaSpawner(BaseSpawner):
    """Real Llanta spawner"""

    def _trail_logs(self, llanta_id: bytes, process: subprocess.Popen):
        """Trail the logs of a started process
        (this method blocks, so start it in a separate thread)
        """
        while True:
            output = process.stdout.readline()
            if output == "" and process.poll() is not None:
                break
            if output:
                _logger.info(f"{llanta_id}: {output.strip()}")

    def spawn_llanta(
        self,
        llanta_id: bytes,
        nta_id: bytes,
        llanta_manager_endpoint_host: str,
        llanta_manager_endpoint_port: int,
        nta_alerts_endpoint_host: str,
        nta_alerts_endpoint_port: int,
        last_message: bytes,
    ) -> Llanta:
        """Start a Llanta process by calling the llanta-small script
        and providing the required params.
        """
        _logger.info(f"Starting new llanta with id: {llanta_id}")

        p = subprocess.Popen(
            [
                "/llanta-threaded-service",
                "-c",
                "/tmp/etc/lastline/llanta.ini",
                "--llanta-id",
                llanta_id,
                "--llanta-manager-endpoint",
                f"tcp://{llanta_manager_endpoint_host}:{llanta_manager_endpoint_port}",
                "--nta-alerts-endpoint",
                f"tcp://{nta_alerts_endpoint_host}:{nta_alerts_endpoint_port}",
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        )

        # log trailing thread
        Thread(target=self._trail_logs, args=(llanta_id, p), daemon=True).start()

        p = psutil.Process(p.pid)
        return Llanta(
            llanta_id=llanta_id, nta_id=nta_id, process=p, last_message=last_message
        )


class MockSpawner(BaseSpawner):
    """Mock Llanta spawner using multiprocessing and the LlantaMock class"""

    def spawn_llanta(
        self,
        llanta_id: bytes,
        nta_id: bytes,
        llanta_manager_endpoint_host: str,
        llanta_manager_endpoint_port: int,
        nta_alerts_endpoint_host: str,
        nta_alerts_endpoint_port: int,
        last_message: bytes,
    ) -> Llanta:
        _logger.info(f"Starting new mock llanta with id: {llanta_id}")

        new_llanta = LlantaMock(llanta_id)
        p = Process(
            target=new_llanta.start,
            args=(
                f"tcp://{llanta_manager_endpoint_host}:{llanta_manager_endpoint_port}",
                f"tcp://{nta_alerts_endpoint_host}:{nta_alerts_endpoint_port}",
            ),
        )
        p.start()

        p = psutil.Process(p.pid)
        return Llanta(
            llanta_id=llanta_id, nta_id=nta_id, process=p, last_message=last_message
        )
