import logging
import time
from proto_proto_pscr.cb_psc_nta_pb2 import NtaRequest, NtaResponse, QueryStatusRequest

_logger = logging.getLogger(__name__)


class SubnetsMonitor:
    """Monitor the active subnets in each llanta process
    by querying each on a given interval
    """

    def __init__(self, llanta_pool, zmq_connector, subnets_request_interval_seconds=90):
        self.llanta_pool = llanta_pool
        self.zmq_connector = zmq_connector
        self.subnets_request_interval_seconds = subnets_request_interval_seconds

    def monitor_llanta(self, llanta_id: bytes):
        """Query for the active subnets of the given llanta
        if the request interval time has passed, since the last request
        """
        if (
            time.time() - self.llanta_pool[llanta_id].last_subnets_request
        ) >= self.subnets_request_interval_seconds:
            self._query_active_subnets(llanta_id)

    def process_active_subnets_response(self, llanta_id: bytes, response: bytes):
        """Unpack the protobuf and update the active_subnets field of the llanta"""
        self.llanta_pool[llanta_id].set_subnets_requested(False)
        decoded_response = NtaResponse()
        decoded_response.ParseFromString(response)
        self.llanta_pool[llanta_id].active_subnets = decoded_response.license_infos

    def _query_active_subnets(self, llanta_id: bytes):
        """Send a request for the active subnets in a given llanta process"""
        if self.llanta_pool[llanta_id].ready:
            _logger.debug(f"Requesting subnets status for {llanta_id}")
            subnets_request = NtaRequest(
                **{"query_status_request": QueryStatusRequest()}
            )
            query_message = [llanta_id, b"", subnets_request.SerializeToString()]
            self.llanta_pool[llanta_id].set_ready(False)
            self.llanta_pool[llanta_id].set_subnets_requested(True)
            self.zmq_connector.send_to_llanta(query_message)
        else:
            _logger.warning(
                f"Llanta wasn't ready, couldn't send active subnets request to {llanta_id}"
            )
