import os
import logging
import signal
import time
from threading import Thread
from typing import Callable
from pscr_common.config import Config as _Config

_logger = logging.getLogger(__name__)


class SigHandler:
    """Handler for interrupts"""

    def __init__(self):
        self.interrupted = False

        signal.signal(signal.SIGINT, self.exit_gracefully)
        signal.signal(signal.SIGTERM, self.exit_gracefully)

    def exit_gracefully(self, *args):
        self.interrupted = True


class Config(_Config):
    CONFIG_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "config.ini")

    def print_config_props(self):
        # report settings for llanta manager
        if self.getboolean("service", "config_show"):
            _logger.info(f"Configuration properties: { {section: dict(self[section]) for section in self.sections()} }")

    def get_special(self, section, option, **kwargs):
        """Properly handle backslashes in .ini values"""
        return self.get(section, option, **kwargs).encode().decode("unicode_escape")


class ScheduledWorkerThread(Thread):
    """Worker thread that runs a given job on a schedule"""

    def __init__(self, job: Callable, job_interval_seconds: int):
        super().__init__(daemon=True)
        self.job = job
        self.job_interval_seconds = job_interval_seconds

    def job_schedule(self):
        """Run job on a schedule"""
        while True:
            time.sleep(self.job_interval_seconds)
            try:
                self.job()
            except Exception as e:
                _logger.error(
                    f"Exception occured while running scheduled job {self.job.__name__}: \n{e}"
                )

    def run(self):
        self.job_schedule()
