import zmq
from typing import List, Tuple


class ZmqConnector:
    """Class to handle communication through ZMQ"""

    def __init__(self, nta_endpoint_port, llanta_endpoint_port, poll_timeout):
        self.nta_endpoint_port = nta_endpoint_port
        self.llanta_endpoint_port = llanta_endpoint_port
        self.poll_timeout = poll_timeout

        self.ctx = zmq.Context()
        self.nta_socket = self.ctx.socket(zmq.ROUTER)
        self.llanta_socket = self.ctx.socket(zmq.ROUTER)

        self.poller = zmq.Poller()
        self.poller.register(self.nta_socket, zmq.POLLIN)
        self.poller.register(self.llanta_socket, zmq.POLLIN)

    def bind(self):
        """Bind the sockets for both NTAs and llantas on given TCP ports"""
        self.nta_socket.bind(f"tcp://*:{self.nta_endpoint_port}")
        self.llanta_socket.bind(f"tcp://*:{self.llanta_endpoint_port}")

    def close_sockets(self):
        """Close the sockets before shutting down"""
        self.nta_socket.setsockopt(zmq.LINGER, 0)
        self.nta_socket.close()

        self.llanta_socket.setsockopt(zmq.LINGER, 0)
        self.llanta_socket.close()

    def poll_sockets(self) -> Tuple[bool, bool]:
        """Check if we've got new messages on both sockets"""
        sockets = dict(self.poller.poll(self.poll_timeout))

        got_llanta_messages = sockets.get(self.llanta_socket) == zmq.POLLIN
        got_nta_messages = sockets.get(self.nta_socket) == zmq.POLLIN

        return (got_llanta_messages, got_nta_messages)

    def read_nta_message(self) -> Tuple[bytes, bytes]:
        nta_id, _, message = self.nta_socket.recv_multipart()
        return (nta_id, message)

    def read_llanta_message(self) -> Tuple[bytes, bytes]:
        llanta_id, _, message = self.llanta_socket.recv_multipart()
        return (llanta_id, message)

    def send_to_llanta(self, message: List[bytes]):
        self.llanta_socket.send_multipart(message)

    def send_to_nta(self, message: List[bytes]):
        self.nta_socket.send_multipart(message)
